@@ -1,6 +1,5 @@
 import math
-import warnings
-from random import randrange
+from random import randint
 from time import time
 from typing import Any, Callable, List, Tuple
 
@@ -14,13 +13,13 @@ class bigO:
     Methods
     -------
     test(function, str):
-        Returns the complexity and the execution time to sort arrays with function
+        Returns time complexity and the execution time to sort arrays with your function
 
     Usage
     -----
         from bigO import bigO
         bigO = bigO.bigO()
-        bigO.test(someSort, "random")
+        bigO.test(mySort, "random")
     """
 
     def __init__(self):
@@ -133,32 +132,26 @@ class bigO:
 
         return bestFit
 
-    def genRandomArray(_, size):
-        array = []
-        for _ in range(size):
-            array.append(randrange(-size, size))
+    def genRandomArray(_, size: int):
+        array = [randint(-size, size) for i in range(size)]
         return array
 
-    def genSortedArray(_, size):
-        array = []
-        for value in range(size):
-            array.append(value)
+    def genSortedArray(_, size: int):
+        array = [i for i in range(size)]
         return array
 
-    def genReversedArray(_, size):
-        array = []
-        for value in reversed(range(size)):
-            array.append(value)
+    def genReversedArray(_, size: int):
+        array = [i for i in reversed(range(size))]
         return array
 
-    def genPartialArray(self, size):
+    def genPartialArray(self, size: int):
         array = self.genRandomArray(size)
         sorted_array = self.genSortedArray(size)
 
         array[size // 4 : size // 2] = sorted_array[size // 4 : size // 2]
         return array
 
-    def genKsortedArray(self, size, k):
+    def genKsortedArray(self, size: int, k: int = None):
         def reverseRange(array, a, b):
             i = a
             j = b - 1
@@ -169,35 +162,36 @@ class bigO:
 
             return array
 
+        if k is None:
+            k = size.bit_length()
+
         assert size >= k, "K must be smaller than the size."
-        if size == 0:
+        if k == 0:
             return self.genSortedArray(size)
         elif size == k:
             return self.genReversedArray(size)
 
-        array = []
-        for value in range(-size // 2, size // 2):
-            array.append(value)
+        array = [value for value in range(-size // 2, size // 2)]
 
-        right = randrange(k)
-        while right > size - k - 1:  # Don't reverse again
+        right = randint(0, k - 1)
+        while right >= size - k:
             right -= 1
 
-        if size != k:
-            k += 1
-
-        reverseRange(array, 0, k)
-        reverseRange(array, len(array) - right, len(array))
+        reverseRange(array, 0, k + 1)
+        reverseRange(array, size - right, size)
 
         return array
 
-    def test(self, functionName: Callable, array: str) -> Tuple[str, float]:
+    def test(
+        self, functionName: Callable, array: str, limit: bool = True
+    ) -> Tuple[str, float]:
         """
         ex) test(bubbleSort, "random")
 
         Args:
             functionName (Callable): a function to call |
-            array (str): "random", "sorted", "reversed", "partial", "Ksorted"
+            array (str): "random", "sorted", "reversed", "partial", "Ksorted" |
+            limit (bool): To terminate before it takes forever to sort (usually 10,000)
 
         Returns:
             complexity (str) : ex) O(n) |
@@ -210,6 +204,7 @@ class bigO:
         sizes = [10, 100, 1000, 10000, 100000]
         maxIter = 5
         times = []
+        isSlow = False  # To see if sorting algorithm takes forever
 
         toaster = ToastNotifier()
         print(f"Running {functionName.__name__}({array} array)...")
@@ -220,6 +215,11 @@ class bigO:
         )
 
         for size in sizes:
+
+            if isSlow:
+                sizes = sizes[: len(times)]
+                break
+
             timeTaken = 0.0
             nums = []
 
@@ -233,7 +233,7 @@ class bigO:
             elif array == "reversed":
                 nums = self.genReversedArray(size)
             elif array == "ksorted":
-                nums = self.genKsortedArray(size, size // 10)
+                nums = self.genKsortedArray(size, size.bit_length())
             # elif array == "custom":
             #    nums = custom
             #    assert len(nums) != 0, "Please, pass the custom array you want.
@@ -254,6 +254,11 @@ class bigO:
                         nums
                     ), "This function doesn't sort correctly."
 
+            if (
+                timeTaken >= 5 and limit
+            ):  # if it takes more than 5 seconds to sort one array, break
+                isSlow = True
+
             timeTaken /= maxIter
             times.append(timeTaken)
 
